// ExportModelDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Hxt.h"
#include "ExportModelDialog.h"
#include "F_Protos.h"
#include "MapLimit.h"

#include "SDK\HModel.h"
#include "SDK\Misc.h"

#include <string.h>
#include <atlbase.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL skinned_export;

/////////////////////////////////////////////////////////////////////////////
// CExportModelDialog dialog

CExportModelDialog::CExportModelDialog(const String &filename)
: CDialog(CExportModelDialog::IDD, CWnd::FromHandle(GetMainApplicationWnd()))
{


#ifdef AFXMANAGESTATE
AFX_MANAGE_STATE(AfxGetStaticModuleState());
#endif

   String path = GetLocalRegistryString("AMXtex .X Plugin", "File Path", ".");
   m_filename = filename.GetNameOnlyWithoutSuffix() + ".X";
   m_filename = CombineNameToPath(path, m_filename);
   m_subtype = GetLocalRegistryValue("AMXtex .X Plugin", "Sub Type", (int)ST_1);

   //{{AFX_DATA_INIT(CExportModelDialog)
	m_expboneinfo = FALSE;
	m_expmatinfo = FALSE;
	m_expnorminfo = FALSE;
	m_exptexinfo = FALSE;
	m_expvdis = FALSE;
	m_exportstyle = -1;
	m_modelorientation = -1;
	m_alphasopaque = FALSE;
	m_settextowhite = FALSE;
	m_texturesort = FALSE;
	m_ignoreduplicatenormals = FALSE;
	m_expweightinfo = FALSE;
	//}}AFX_DATA_INIT

	m_generatemodelmaps = GetLocalRegistryString("AMXtex .X Plugin", "Generate Model Maps", "FALSE")=="TRUE";
	m_maxsize = GetLocalRegistryValue("AMXtex .X Plugin", "Max Map Size", (int)ML_256);
	m_minsize = GetLocalRegistryValue("AMXtex .X Plugin", "Min Map Size", (int)ML_0);

	m_exptexinfo = GetLocalRegistryString("AMXtex .X Plugin", "Export Texture Info", "TRUE")=="TRUE";
	m_settextowhite = GetLocalRegistryString("AMXtex .X Plugin", "Set Tex to White", "TRUE")=="TRUE";
	m_texturesort = GetLocalRegistryString("AMXtex .X Plugin", "Texture Sort", "FALSE")=="TRUE";
	m_expweightinfo = GetLocalRegistryString("AMXtex .X Plugin", "Export Weight Info", "TRUE")=="TRUE";

	m_exportstyle = GetLocalRegistryValue("AMXtex .X Plugin", "DirectX Type", (int)DX_SM);

	if (m_exportstyle == 0) // skinned mesh
		skinned_export = true;
	else					// retained mesh
		skinned_export = false;

	m_modelorientation = GetLocalRegistryValue("AMXtex .X Plugin", "Model Orientation", (int)DX_RIGHTHANDED);

	m_expboneinfo = GetLocalRegistryString("AMXtex .X Plugin", "Export Bone Info", "TRUE")=="TRUE";
	m_expvdis = GetLocalRegistryString("AMXtex .X Plugin", "Export VDIs", "TRUE")=="TRUE";
	m_ignoreduplicatenormals = GetLocalRegistryString("AMXtex .X Plugin", "Ignore Duplicate Normals", "TRUE")=="TRUE";
	m_expnorminfo = GetLocalRegistryString("AMXtex .X Plugin", "Export Normal Info", "TRUE")=="TRUE";
	m_expmatinfo = GetLocalRegistryString("AMXtex .X Plugin", "Export Material Info", "TRUE")=="TRUE";
	m_alphasopaque = GetLocalRegistryString("AMXtex .X Plugin", "Opaque Alphas", "FALSE")=="TRUE";
}

void CExportModelDialog::FillCombo(int id)
{
   int index;
   CComboBox *combo = (CComboBox *)GetDlgItem(id);
   index = combo->AddString("0");
   combo->SetItemData(index, (DWORD) ML_0);

   index = combo->AddString("1");
   combo->SetItemData(index, (DWORD) ML_1);

   index = combo->AddString("2");
   combo->SetItemData(index, (DWORD) ML_2);

   index = combo->AddString("4");
   combo->SetItemData(index, (DWORD) ML_4);

   index = combo->AddString("8");
   combo->SetItemData(index, (DWORD) ML_8);

   index = combo->AddString("16");
   combo->SetItemData(index, (DWORD) ML_16);

   index = combo->AddString("32");
   combo->SetItemData(index, (DWORD) ML_32);

   index = combo->AddString("64");
   combo->SetItemData(index, (DWORD) ML_64);

   index = combo->AddString("128");
   combo->SetItemData(index, (DWORD) ML_128);

   index = combo->AddString("256");
   combo->SetItemData(index, (DWORD) ML_256);
}

BOOL CExportModelDialog::OnInitDialog() 
{
   CWnd *dlgitem = GetDlgItem(IDC_FILENAME);
   String text;
   text = m_filename.GetAbbreviated(dlgitem);
   dlgitem->SetWindowText(text.Get());
   CString WindowTitle;
   CString Username;

   FillCombo(IDC_MINSIZE_COMBO);
   FillCombo(IDC_MAXSIZE_COMBO);
   CDialog::OnInitDialog();

   GetDlgItem(IDC_MINSIZE_COMBO)->EnableWindow(m_generatemodelmaps);
   GetDlgItem(IDC_MAXSIZE_COMBO)->EnableWindow(m_generatemodelmaps);

	// if either expvdis or expnorminfo is false, disable ingoredupenorms
	// if they're both true, enable the window
	m_expvdis = IsDlgButtonChecked(IDC_EXPVDIS);
	m_expnorminfo = IsDlgButtonChecked(IDC_EXPNORMINFO);

	if (!m_expvdis || !m_expnorminfo) {
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(FALSE);
		CheckDlgButton(IDC_IGNOREDUPLICATENORMALS, BST_UNCHECKED);
		m_ignoreduplicatenormals = FALSE;
	}
	else
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(TRUE);

	// if not exporting mat info, gray out the opaque alpha box
	if (!m_expmatinfo) {
		GetDlgItem(IDC_SETALPHASOPAQUE)->EnableWindow(FALSE);
		CheckDlgButton(IDC_SETALPHASOPAQUE, BST_UNCHECKED);
		m_alphasopaque = FALSE;
	}
	else
		GetDlgItem(IDC_SETALPHASOPAQUE)->EnableWindow(TRUE);

	// check if retained mode is active, and if so, block out bone option
	if ((int)m_exportstyle == (int)DX_RM) {
		CheckDlgButton(IDC_EXPBONEINFO, BST_CHECKED);
		GetDlgItem(IDC_EXPBONEINFO)->EnableWindow(FALSE);
		m_expboneinfo = TRUE;
		GetDlgItem(IDC_EXPWEIGHTINFO)->EnableWindow(FALSE);
		m_expweightinfo = FALSE;
	}

   return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CExportModelDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExportModelDialog)
	DDX_Check(pDX, IDC_GENERATEMODELMAPS, m_generatemodelmaps);
	DDX_CBIndex(pDX, IDC_MAXSIZE_COMBO, m_maxsize);
	DDX_CBIndex(pDX, IDC_MINSIZE_COMBO, m_minsize);
	DDX_Radio(pDX, IDC_SUB1, m_subtype);
	DDX_Check(pDX, IDC_EXPBONEINFO, m_expboneinfo);
	DDX_Check(pDX, IDC_EXPMATINFO, m_expmatinfo);
	DDX_Check(pDX, IDC_EXPNORMINFO, m_expnorminfo);
	DDX_Check(pDX, IDC_EXPTEXINFO, m_exptexinfo);
	DDX_Check(pDX, IDC_EXPVDIS, m_expvdis);
	DDX_Radio(pDX, IDC_RADIO1, m_exportstyle);
	DDX_Radio(pDX, IDC_RADIO3, m_modelorientation);
	DDX_Check(pDX, IDC_SETALPHASOPAQUE, m_alphasopaque);
	DDX_Check(pDX, IDC_SETTEXTOWHITE, m_settextowhite);
	DDX_Check(pDX, IDC_TEXTURESORT, m_texturesort);
	DDX_Check(pDX, IDC_IGNOREDUPLICATENORMALS, m_ignoreduplicatenormals);
	DDX_Check(pDX, IDC_EXPWEIGHTINFO, m_expweightinfo);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CExportModelDialog, CDialog)
	//{{AFX_MSG_MAP(CExportModelDialog)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_CBN_SELCHANGE(IDC_MAXSIZE_COMBO, OnMaxSize)
	ON_CBN_SELCHANGE(IDC_MINSIZE_COMBO, OnMinSize)
	ON_BN_CLICKED(IDC_GENERATEMODELMAPS, OnGeneratemodelmaps)
	ON_BN_CLICKED(IDC_EXPMATINFO, OnExpmatinfo)
	ON_BN_CLICKED(IDC_SETALPHASOPAQUE, OnSetalphasopaque)
	ON_BN_CLICKED(IDC_EXPNORMINFO, OnExpnorminfo)
	ON_BN_CLICKED(IDC_EXPVDIS, OnExpvdis)
	ON_BN_CLICKED(IDC_EXPTEXINFO, OnExptexinfo)
	ON_BN_CLICKED(IDC_SETTEXTOWHITE, OnSettextowhite)
	ON_BN_CLICKED(IDC_TEXTURESORT, OnTexturesort)
	ON_BN_CLICKED(IDC_EXPBONEINFO, OnExpboneinfo)
	ON_BN_CLICKED(IDABOUT, OnAbout)
	ON_BN_CLICKED(IDC_IGNOREDUPLICATENORMALS, OnIgnoreduplicatenormals)
	ON_BN_CLICKED(IDC_RADIO2, OnRadio2)
	ON_BN_CLICKED(IDC_RADIO1, OnRadio1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExportModelDialog message handlers

void CExportModelDialog::OnOK() 
{
	CDialog::OnOK();

#ifdef AFXMANAGESTATE
AFX_MANAGE_STATE(AfxGetStaticModuleState());
#endif

   String path = m_filename.GetPathOnly();
   SetLocalRegistryString("AMXtex .X Plugin", "File Path", path );
   SetLocalRegistryString("AMXtex .X Plugin", "Generate Model Maps", m_generatemodelmaps?"TRUE":"FALSE");
   SetLocalRegistryValue("AMXtex .X Plugin", "Min Map Size", m_minsize);
   SetLocalRegistryValue("AMXtex .X Plugin", "Max Map Size", m_maxsize);
   SetLocalRegistryValue("AMXtex .X Plugin", "Sub Type", (int)m_subtype);
   SetLocalRegistryValue("AMXtex .X Plugin", "DirectX Type", (int)m_exportstyle);
   SetLocalRegistryValue("AMXtex .X Plugin", "Model Orientation", (int)m_modelorientation);

   SetLocalRegistryString("AMXtex .X Plugin", "Export Bone Info", m_expboneinfo?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Export VDIs", m_expvdis?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Ignore Duplicate Normals", m_ignoreduplicatenormals?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Export Normal Info", m_expnorminfo?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Export Material Info", m_expmatinfo?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Opaque Alphas", m_alphasopaque?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Export Texture Info", m_exptexinfo?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Set Tex to White", m_settextowhite?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Texture Sort", m_texturesort?"TRUE":"FALSE");
   SetLocalRegistryString("AMXtex .X Plugin", "Export Weight Info", m_expweightinfo?"TRUE":"FALSE");

   // this check should never happen anymore (2.09 fix), but leaving it just in case
   if ((m_exportstyle == (int)DX_RM) &&
	   (!m_expboneinfo)) {
	   //CString msg, title;
	   //msg = "Bone export will be enabled; It's necessary for retained mode export";
	   //title = "AMXtex notification";
	   //MessageBox(GetMainApplicationWnd(), msg, title, MB_OK);
	   //MessageBox(GetMainApplicationWnd(), "Bone export will be enabled; It's neccessary for Retained Mode export", "AMXtex Notification", MB_OK);
	   AfxMessageBox("Bone export will be enabled;\nIt's necessary for retained mode export", MB_ICONINFORMATION);
	   m_expboneinfo = TRUE;
	   SetLocalRegistryString("AMXtex .X Plugin", "Export Bone Info", m_expboneinfo?"TRUE":"FALSE");
   }

}

void CExportModelDialog::OnBrowse() 
{
   CString filters, title;
   title.LoadString(IDS_EXPORTTITLE);
   filters.LoadString(IDS_FILEFILTERS);

   String ext("X");
   if (!SaveRequest((LPCTSTR)title, (LPCTSTR)filters, m_filename, ext.Get(), ext.Get()))
      return;
   
   CWnd *dlgitem = GetDlgItem(IDC_FILENAME);
   String text;
   text = m_filename.GetAbbreviated(dlgitem);
   dlgitem->SetWindowText(text.Get());
}

void CExportModelDialog::OnMaxSize() 
{
   CComboBox *mincombo = (CComboBox *)GetDlgItem(IDC_MINSIZE_COMBO);
   CComboBox *maxcombo = (CComboBox *)GetDlgItem(IDC_MAXSIZE_COMBO);
   
   int index = maxcombo->GetCurSel();
   int maxlimit = maxcombo->GetItemData(index);

   index = mincombo->GetCurSel();
   int minlimit = mincombo->GetItemData(index);
   
   
   if (maxlimit < minlimit)
      maxcombo->SetCurSel(index);
}

void CExportModelDialog::OnMinSize() 
{
   CComboBox *mincombo = (CComboBox *)GetDlgItem(IDC_MINSIZE_COMBO);
   CComboBox *maxcombo = (CComboBox *)GetDlgItem(IDC_MAXSIZE_COMBO);
   
   int index = mincombo->GetCurSel();
   int minlimit = mincombo->GetItemData(index);
   
   index = maxcombo->GetCurSel();
   int maxlimit = maxcombo->GetItemData(index);
   
   if (minlimit > maxlimit)
      mincombo->SetCurSel(index);
}

void CExportModelDialog::OnGeneratemodelmaps() 
{
   m_generatemodelmaps = IsDlgButtonChecked(IDC_GENERATEMODELMAPS);
   GetDlgItem(IDC_MINSIZE_COMBO)->EnableWindow(m_generatemodelmaps);
   GetDlgItem(IDC_MAXSIZE_COMBO)->EnableWindow(m_generatemodelmaps);
}

void CExportModelDialog::OnExpmatinfo() 
{
	// Check the state of the material and texture checkboxes
	m_expmatinfo = IsDlgButtonChecked(IDC_EXPMATINFO);
	//m_exptexinfo = IsDlgButtonChecked(IDC_EXPTEXINFO);

	GetDlgItem(IDC_SETALPHASOPAQUE)->EnableWindow(m_expmatinfo);

	// set the same state for the texture window (ie if the 
	// material box is unchecked, disable the texture window)
	GetDlgItem(IDC_TEXTUREBOX)->EnableWindow(m_expmatinfo);
	GetDlgItem(IDC_EXPTEXINFO)->EnableWindow(m_expmatinfo);
	GetDlgItem(IDC_SETTEXTOWHITE)->EnableWindow(m_expmatinfo);
	GetDlgItem(IDC_TEXTURESORT)->EnableWindow(m_expmatinfo);

	// should this be above the enable window logically?
	// Anyway, this unchecks the texture box if it grays out 
	//   if (m_exptexinfo && !m_expmatinfo)
	if (!m_expmatinfo) {
		CheckDlgButton(IDC_EXPTEXINFO, BST_UNCHECKED);
		CheckDlgButton(IDC_SETTEXTOWHITE, BST_UNCHECKED);
		CheckDlgButton(IDC_TEXTURESORT, BST_UNCHECKED);
	}

	OnExptexinfo();

	if (!m_expmatinfo)
		CheckDlgButton(IDC_SETALPHASOPAQUE, BST_UNCHECKED);
	
}

void CExportModelDialog::OnSetalphasopaque() 
{
	// TODO: Add your control notification handler code here
	
}

void CExportModelDialog::OnExpnorminfo() 
{
	// if either expvdis or expnorminfo is false, disable ingoredupenorms
	// if they're both true, enable the window
	m_expvdis = IsDlgButtonChecked(IDC_EXPVDIS);
	m_expnorminfo = IsDlgButtonChecked(IDC_EXPNORMINFO);

	if (!m_expvdis || !m_expnorminfo) {
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(FALSE);
		CheckDlgButton(IDC_IGNOREDUPLICATENORMALS, BST_UNCHECKED);
		m_ignoreduplicatenormals = FALSE;
	}
	else
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(TRUE);
	
}

void CExportModelDialog::OnExpvdis() 
{
	// if either expvdis or expnorminfo is false, disable ingoredupenorms
	// if they're both true, enable the window
	m_expvdis = IsDlgButtonChecked(IDC_EXPVDIS);
	m_expnorminfo = IsDlgButtonChecked(IDC_EXPNORMINFO);

	if (!m_expvdis || !m_expnorminfo) {
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(FALSE);
		CheckDlgButton(IDC_IGNOREDUPLICATENORMALS, BST_UNCHECKED);
		m_ignoreduplicatenormals = FALSE;
	}
	else
		GetDlgItem(IDC_IGNOREDUPLICATENORMALS)->EnableWindow(TRUE);

}

void CExportModelDialog::OnExptexinfo() 
{
	// Check the state of the texture checkboxes
	m_exptexinfo = IsDlgButtonChecked(IDC_EXPTEXINFO);

	// set the same state for the texture window (ie if the 
	// material box is unchecked, disable the texture window)
	GetDlgItem(IDC_SETTEXTOWHITE)->EnableWindow(m_exptexinfo);
	GetDlgItem(IDC_TEXTURESORT)->EnableWindow(m_exptexinfo);

	// should this be above the enable window logically?
	// Anyway, this unchecks the texture box if it grays out 
	//   if (m_exptexinfo && !m_expmatinfo)
	if (!m_exptexinfo) {
		CheckDlgButton(IDC_SETTEXTOWHITE, BST_UNCHECKED);
		CheckDlgButton(IDC_TEXTURESORT, BST_UNCHECKED);
	}
	
}

void CExportModelDialog::OnSettextowhite() 
{
	// TODO: Add your control notification handler code here
	
}

void CExportModelDialog::OnTexturesort() 
{
	// TODO: Add your control notification handler code here
	
}

void CExportModelDialog::OnExpboneinfo() 
{
	// TODO: Add your control notification handler code here
}

void CExportModelDialog::OnAbout() 
{
	AfxMessageBox("AMXtex DirectX .X Exporter for Animation:Master\nVersion 3.0\nCopyright 2001-2006 Christopher J Roy, Obsidian Games\n\nhttp://www.obsidiangames.com", MB_ICONINFORMATION);

	// TODO: Add your control notification handler code here
	
}

void CExportModelDialog::OnIgnoreduplicatenormals() 
{
	// TODO: Add your control notification handler code here
	
}

void CExportModelDialog::OnRadio2()  // retained mode button
{
	CheckDlgButton(IDC_EXPBONEINFO, BST_CHECKED);
	GetDlgItem(IDC_EXPBONEINFO)->EnableWindow(FALSE);
	m_expboneinfo = TRUE;
	
	GetDlgItem(IDC_EXPWEIGHTINFO)->EnableWindow(FALSE);
	CheckDlgButton(IDC_EXPWEIGHTINFO, BST_UNCHECKED);
	m_expweightinfo = FALSE;

	skinned_export = FALSE;
}

void CExportModelDialog::OnRadio1() // skinned mesh button
{
	GetDlgItem(IDC_EXPBONEINFO)->EnableWindow(TRUE);

	GetDlgItem(IDC_EXPWEIGHTINFO)->EnableWindow(TRUE);

	skinned_export = TRUE;
}

